import { useEventBus } from '@vueuse/core';
import { RecordAuthResponse, RecordSubscription, UnsubscribeFunc } from 'pocketbase';
import { Collections, CompaniesRecord, CompaniesResponse, TransactionsResponse } from '../types/pocketbase.types';
import { AccountService } from './account.service';
import { AuthService } from './auth.service';
import { AccountType, BankService } from './bank.service';
import { PocketbaseService } from './pocketbase.service';
import { SettingsService } from './settings.service';
import { Shifts } from '../enums/shift.enum';

const API = PocketbaseService.getApi();
const COLLECTION = API.collection(Collections.Companies);

export class CompanyService {
  public static async addCompany(company: CompaniesRecord & {password: string}): Promise<CompaniesResponse> {
    return COLLECTION.create({ ...company, passwordConfirm: company.password }, { $cancelKey: `${company.name}` });
  }
  public static async getCompanies(): Promise<CompaniesResponse[]> {
    return COLLECTION.getFullList({ sort: 'name' });
  }
  public static async login(username: string, password: string): Promise<RecordAuthResponse<CompaniesResponse>> {
    try {
      const response = await COLLECTION.authWithPassword<CompaniesResponse>(username, password);
      useEventBus<boolean>('isAdmin').emit(AuthService.isAdmin());
      return response;
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    } catch (error: any) {
      throw new Error(error);
    }
  }
  public static isAuthenticated(): boolean {
    if(API.authStore.isValid && API.authStore.model) {
      return Boolean(API.authStore.model.collectionId);
    }
    this.logout();
    return false;
  }
  public static logout(): void {
    API.authStore.clear();
  }
  public static async getCompany(): Promise<CompaniesResponse> {
    if(this.isAuthenticated()) {
      return COLLECTION.getFirstListItem(`id="${API.authStore.model?.id}"`);
    }
    throw new Error('Not authenticated');
  }
  public static async getCompanyByAccountNumber(accountNumber: string): Promise<CompaniesResponse> {
    return COLLECTION.getFirstListItem(`accountNumber="${accountNumber}"`);
  }
  public static async updateAccountNumber(companyId: string, accountNumber: string): Promise<CompaniesResponse> {
    return COLLECTION.update(companyId, { accountNumber });
  }
  public static async subscribeToCompanyChanges(
    callback: (data: RecordSubscription<CompaniesResponse>)=> void,
  ): Promise<UnsubscribeFunc> {
    return await COLLECTION.subscribe<CompaniesResponse>('*', (data) => {
      callback(data);
    });
  }
  public static async payWage(shift: Shifts): Promise<boolean> {
    const company = await this.getCompany();
    if(company) {
      const settings = await SettingsService.getSettings();
      const balance = (await BankService.getCompanyTransactions(company.id)).reduce((acc, transaction) => acc + transaction.amount, 0);
      const accounts = (await AccountService.getAccountsByCompanyId(company.id)).filter(account => account.shift == shift);
      const totalWages = accounts.reduce((acc, account) => acc + account.wage, 0);
      let taxes = 0;
      if(balance >= totalWages) {
        await BankService.addTransaction(company.id, -totalWages / 100, `Lohn für ${shift}`, AccountType.COMPANY);
        const promises: Promise<TransactionsResponse>[] = [];
        accounts.forEach(async account => {
          promises.push(BankService.addTransaction(
            account.id,
            ((account.wage * (1 - (settings.incomeTax / 100)))) / 100,
            `Lohn für ${shift} bei ${company.name}`,
          ));
          taxes += account.wage * (settings.incomeTax / 100);
        });
        await Promise.all(promises);
        await BankService.addTransaction(settings.incomeTaxRecipient,
          taxes / 100,
          `Lohnsteuer von ${company.name} für ${shift}`,
          AccountType.COMPANY,
        );
        if(shift == Shifts.EARLY) {
          await COLLECTION.update<CompaniesResponse>(company.id, { earlyShiftPayed: new Date() });
        } else {
          await COLLECTION.update(company.id, { lateShiftPayed: new Date() });
        }
        return true;
      }
    }
    return false;
  }
}